/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#ifndef _NEPOMUK_ABSTRACT_RESOURCE_GUI_ITEM_H_
#define _NEPOMUK_ABSTRACT_RESOURCE_GUI_ITEM_H_

#include <QtCore/QObject>
#include <QtCore/QUrl>
#include <QtGui/QStyleOption>

#include <nepomuk/nepomuk_export.h>

class QPainter;
class QEvent;
class QWidget;


namespace Nepomuk {
    /**
     * The base class for objects that are able to visualize
     * a Nepomuk resource and optionally provide GUI interaction
     * like reaction to mouse clicks.
     *
     * Typical usage of these gui items is within ResourceDelegate
     * within QT's model/view system or simply to paint resources
     * into widgets.
     *
     * Gui items are created through the GuiItemFactory which tries
     * to load the gui item that can visualize a certain resource type.
     *
     * GenericResourceGuiItem is able to visualize all kinds of
     * resources in a generic manner using generic properties such as
     * rdfs:label or rdfs:comment.
     *
     * Has not been designed as a QObject so it can be used in multi-
     * inheritance (Does this make sense?).
     *
     * Why not use QAbstractViewItem?
     * - Because it should be possible to visualize resources also in
     *   Widgets and QGraphicsItems (i.e. Plasma).
     *
     * \section plugins Plugin
     *
     * KDE Service type: Nepomuk/ResourceGuiItem
     *
     * X-Nepomuk-ResourceTypes should list the supported resource types.
     */
    class NEPOMUK_EXPORT AbstractResourceGuiItem : public QObject
    {
        Q_OBJECT

    public:
        AbstractResourceGuiItem( QObject* parent = 0 );
        virtual ~AbstractResourceGuiItem();

        // FIXME: what about multi-column views?
        // FIXME: what about editing (for example tags)? Maybe just reproduce the view item interface, too, and ignore plasma here?

        /**
         * The resource can be rendered in different ways with
         * different levels of detail.
         *
         * Implementations can choose to not support all or
         * even ignore the display more alltogether.
         */
        enum DisplayMode {
            DisplaySimple,      /**< The most simple display, i.e. mostly just a short string and maybe a pixmap */
            DisplayOptimal,     /**< Try to choose the optimal amount of information to display to the user. */
            DisplayFull         /**< The probably really big display without any restrictions. */
        };

        /**
         * Mostly used to handle the two special roles and allow default
         * delegates to paint resources.
         *
         * \param resource The resource for which data should be returned
         * \param role The display role.
         */
        virtual QVariant data( const QUrl& resource, int role ) = 0;

        /**
         * Paint the resource.
         *
         * \param painter The painter to paint to.
         * \param mode The display mode to use.
         * \param option The paint options. This can also be a QStyleOptionViewItem
         *               Includes the size.
         * \param resource The resource to visualize as can be found in the Nepomuk data store.
         */
        virtual void paint( QPainter* painter, DisplayMode mode, const QStyleOption& option, const QUrl& resource ) = 0;

        /**
         * The preferred size.
         *
         * \param mode The display mode to use.
         * \param option The paint options. This can also be a QStyleOptionViewItem
         * \param resource The resource to visualize as can be found in the Nepomuk data store.
         */
        virtual QSize sizeHint( DisplayMode mode, const QStyleOption& option, const QUrl& resource ) = 0;

        /**
         * The minimal size.
         *
         * \param mode The display mode to use.
         * \param option The paint options. This can also be a QStyleOptionViewItem
         * \param resource The resource to visualize as can be found in the Nepomuk data store.
         */
        virtual QSize minimumSizeHint( DisplayMode mode, const QStyleOption& option, const QUrl& resource );

        /**
         * Handle an event such as mouse clicks.
         *
         * Default implementation returns false (indicating that it has not handled the event).
         *
         * \param event The event to handle
         * \param resource The resource to visualize as can be found in the Nepomuk data store.
         *
         * \return \p true if the event was handled. Is might be nesessary to repaint the resource after handling
         * and event such as a mouse event since user interaction might have changed the data.
         */
        virtual bool handleEvent( QEvent* event, DisplayMode mode, const QStyleOption& option, const QUrl& resource );

    Q_SIGNALS:
        /**
         * Subclasses may emit this signal to inform that data has been changed and needs to be
         * repainted.
         */
        void dataChanged( const QUrl& uri );

    private:
        class Private;
        Private* const d;
    };
}

#endif
