/*
 * This file is part of Nepomuk-KDE Project.
 *
 * Copyright (C) 2009 Sebastian Trueg <trueg@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef _NEPOMUK_META_ANNOTATION_PLUGIN_H_
#define _NEPOMUK_META_ANNOTATION_PLUGIN_H_

#include "annotationplugin.h"

#include "nepomukannotation_export.h"


namespace Nepomuk {
    /**
     * \class MetaAnnotationPlugin metaannotationplugin.h MetaAnnotationPlugin
     *
     * \brief Base class for plugins that use AnnotationRequest::filter() to request
     * annotation from other plugins.
     *
     * The MetaAnnotationPlugin can be used as a base for plugins that in some way
     * determine a list of keywords and want to use the plugin system again to actually
     * create Annotation objects from those.
     *
     * The typical situation would be a plugin that analyses the AnnotationRequest::text()
     * and then delegates the creation of the actual annotations to other plugins.
     *
     * \section usage Usage
     *
     * As with any AnnotationPlugin AnnotationPlugin::doGetPossibleAnnotations needs
     * to be reimplemented. But instead of emitting new Annotation objects through
     * AnnotationPlugin::addNewAnnotation directly, once calls getPossibleMetaAnnotations
     * to let the other plugins handle it.
     *
     * newMetaAnnotation can be reimplemented to perform whatever task necessary with created
     * Annotation objects. metaAnnotationsFinished needs to be reimplemented.
     *
     * \author Sebastian Trueg <trueg@kde.org>
     */
    class NEPOMUKANNOTATION_EXPORT MetaAnnotationPlugin : public AnnotationPlugin
    {
        Q_OBJECT

    public:
        MetaAnnotationPlugin( QObject* parent = 0 );
        virtual ~MetaAnnotationPlugin();

    protected:
        /**
         * This is the heart of the MetaAnnotationPlugin. It triggers the plugin system
         * to create Annotation objects for \p request.
         *
         * Annotations created by other plugins are reported via newMetaAnnotation. Once done,
         * metaAnnotationsFinished is called.
         */
        void getPossibleMetaAnnotations( const AnnotationRequest& request );

        /**
         * This method is called for each annotation that the plugin system
         * creates. By reimplementing this method it is possible to filter the
         * annotations, change their relevance, wrap them in another Annotation,
         * or do whatever necessary with them
         *
         * The default implementation calls AnnotationPlugin::addNewAnnotation.
         *
         * \param anno The Annotation created by the plugin system
         */
        virtual void newMetaAnnotation( Annotation* anno );

        /**
         * This method is called once the plugin system has finished analysing
         * the filter and the plugin should provide a new keyword or finish
         * its analysis itself.
         *
         * A typical implementation would read the next keyword from a list.
         *
         * The default implementation calls AnnotationPlugin::emitFinished.
         */
        virtual void metaAnnotationsFinished();

    private:
        class Private;
        Private* const d;

        Q_PRIVATE_SLOT( d, void _k_newAnnotation( Nepomuk::Annotation* ) )
        Q_PRIVATE_SLOT( d, void _k_finished() )
    };
}

#endif
