/*
 * This file is part of the Nepomuk KDE project.
 * Copyright (C) 2008 Adrien Bustany <madcat@mymadcat.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef NEPOMUKPROXY_H
#define NEPOMUKPROXY_H

#include <QtCore>
#include <nepomuk/nepomuk_export.h>

namespace Soprano {
	class BindingSet;
	namespace Util {
		class AsyncResult;
	}
}

namespace Nepomuk {


/**
 * \class NepomukProxy nepomukproxy.h NepomukProxy
 *
 * \brief This class is a proxy abstracting the Soprano functions, and making all the
 * operations asynchronous.
 * WARNING : This class is NOT thread safe at all!
 *
 * \author Adrien Bustany <madcat@mymadcat.com>
 */


class NEPOMUK_EXPORT NepomukProxy : public QObject
{

	Q_OBJECT

public:
	~NepomukProxy();
	
	/**
	 * Returns a pointer to the NepomukProxy, or creates a new one if needed
	 */
	static NepomukProxy* getInstance();

	/**
	 * Add a new nco:PersonContact entry into Nepomuk
	 * \param  fullname The full name of the person to add
	 * \param  group If not QString(), the person will be put in the group with this id
	 * \return The resource id of the new contact
	 * Warning : this function does not check for duplicates, you can have two resources with the same fullname
	 * TODO : Add support for putting the contact in more that one group (not needed for now)
	 */
	QString addContact(const QString& fullname, const QString& group = QString());

	/*
	 * Add a new nco:ContactGroup into Nepomuk
	 * \param groupname The name of the group to add
	 * \return The resource id of the new group
	 * Warning : this function does not check for duplicates, you can have two resources with the same group name
	 */
	QString addGroup(const QString& groupname);

	/**
	 * Creates a new region
	 * \param res The URI of the image resource
	 * \param rect The region rectangle
	 * \param uid  The id of the resource linked with the region
	 */
	void addRegion(const QUrl& res, const QRectF& rect, const QUrl& uid);

	/**
	 * Updates an existing region
	 * \param mid  The Nepomuk id of the region
	 * \param rect The region rectangle
	 */
	void updateRegion(const QString& mid, const QRectF& rect, const QString& uid);

	/**
	 * Removes an existing region from Nepomuk, and unlinks it from other resources
	 * \param mid  The Nepomuk id of the region
	 */
	void removeRegion(const QString& mid);


public Q_SLOTS:
	/**
	 * Retrieves all the nco:PersonContact resources asynchronously, emits the %contacts signal when done
	 * \param group if not QString(), then only the contacts from the given group will be returned. group is a resource uri , not a group name.
	 */
	void getContacts(const QString& group = QString());

	/**
	 * Retrieves all the nco:ContactGroup resources asynchronously, emits the %groups signal when done
	 */
	void getGroups();

	/**
	 * Retrieves all the regions defined for the given file
	 */
	void getRegions(const QString& path);

signals:
	/**
	 * Emitted after the %getContacts query finishes
	 * The following elements are present (but can be empty for gid and gname) in each set : <br>
	 * uid   : nepomuk id of the contact <br>
	 * gid   : id of the group, if the contact is in a group <br>
	 * name  : full name of the contact <br>
	 * gname : name of the group, if the contact is in a group
	 */
	void contacts(const QList<Soprano::BindingSet>& contacts);

	/**
	 * Emitted after the %getGroups query finishes <br>
	 * The following elements are present in each set : <br>
	 * gid   : id of the group, if the contact is in a group <br>
	 * name  : name of the group
	 */
	void groups(const QList<Soprano::BindingSet>& groups);

	/**
	 * Emitted after the %getMarquees query finishes <br>
	 * The following elements are present in each set : <br>
	 * mid    : marquee identifier <br>
	 * uid    : identifier of the resource linked to the region <br>
	 * coords : coordinates of the region, comma separated list containing x, y, width, height (only rectangles are supported for now) <br>
	 * name   : full name of the resource linked to the region
	 */
	void regions(const QList<Soprano::BindingSet>& regions);

	/**
	 * Emitted when addContact finishes. If several addContact were fired, then emitted when all are finished. <br>
	 * Warning : there's no error reporting with this function
	 */
	void contactAdded();

	/**
	 * Emitted when addGroup finishes. If several addGroup were fired, then emitted when all are finished. <br>
	 * Warning : there's no error reporting with this function
	 */
	void groupAdded();

	/**
	 * Emitted when addRegion finishes. <br>
	 * Warning : there's no error reporting with this function
	 */
	void regionAdded();

	/**
	 * Emitted when updateRegion finishes. <br>
	 * Warning : there's no error reporting with this function
	 */
	void regionUpdated();

	/**
	 * Emitted when removeRegion finishes. <br>
	 * Warning : there's no error reporting with this function
	 */
	void regionRemoved();

private:
	/**
	 * Private constructor, use getInstance()
	 */
	NepomukProxy();
	class Private;
	static NepomukProxy*	m_instance;
	Private* d;

private Q_SLOTS:
	void contacts_cb(Soprano::Util::AsyncResult* result);
	void groups_cb(Soprano::Util::AsyncResult* result);
	void regions_cb(Soprano::Util::AsyncResult* result);
	void addContact_cb(Soprano::Util::AsyncResult* result);
	void addGroup_cb(Soprano::Util::AsyncResult* result);
	void addRegion_cb(Soprano::Util::AsyncResult* result);
	void finishUpdate(Soprano::Util::AsyncResult* oldresult);
	void updateRegion_cb(Soprano::Util::AsyncResult* result);
	void removeRegion_cb(Soprano::Util::AsyncResult* result);

};

} // End namespace Nepomuk

#endif
