/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_GRAPHCOMPLETION_H_
#define _SEMCLIP_GRAPHCOMPLETION_H_

#include <QtCore/QSet>

class QString;
class QUrl;

namespace Soprano
{
    class Model;
}

namespace SemClip
{

    /**
     * \class GraphCompletion graphcompletion.h <semclip/graphcompletion.h>
     *
     * \brief Abstract base class for graph completion services.
     *
     * GraphCompletion defines the interface for graph completion services.
     * It consists of a single complete function, which takes a RDF graph
     * (either in form of a RDF/XML serialization or a Soprano model), a set
     * of completion URIs and (as the graph translator) a set of target
     * ontologies as parameters.
     *
     * There are two possible modes of completion. In the first, the supplied
     * URIs are treated as resources which the client wants to have additional
     * information about. In the second, the URIs specify properties whose
     * object resources should be completed if possible.
     *
     * If the newly retrieved graph fragments contain ontologies outside of the
     * supplied target ontologies they are turned over to the graph translator.
     * Newly discovered information is finally added to the original graph, in
     * other words the supplied Soprano model is augmented or the serialization
     * replaced with one corresponding to the combined graph.
     *
     * \sa CompletionMode
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class GraphCompletion
    {
    public:
        virtual ~GraphCompletion()
        {
        }

        /**
         * Modes of completion for the completion service.
         */
        enum CompletionMode {
            CompleteByResource,  /**< Complete given resources directly. */
            CompleteByProperty   /**< Complete objects of given properties. */
        };

        virtual void complete( Soprano::Model* model, const QSet< QUrl >& completionGoals,
                               const QSet< QUrl >& targetOntologies, CompletionMode mode ) = 0;

        virtual void complete( QString& serialization, const QSet< QUrl >& completionGoals,
                               const QSet< QUrl >& targetOntologies, CompletionMode mode ) = 0;
    };

}

#endif
