/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_GRAPHCOMPLETOR_H_
#define _SEMCLIP_GRAPHCOMPLETOR_H_

#include "graphcompletion.h"
#include "semclip_export.h"

#include <QtCore/QObject>

namespace SemClip
{

    class GraphCompletorHelper;

    /**
     * \class GraphCompletor graphcompletor.h <semclip/graphcompletor.h>
     *
     * \brief Implements the SemClip graph completion service.
     *
     * GraphCompletor inherits from GraphCompletion and is a singleton.
     * Clients can access this class directly, but are encouraged to use
     * the SemanticClipboard facade.
     *
     * \sa GraphCompletion, SemanticClipboard
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SEMCLIP_EXPORT GraphCompletor : public QObject,
                                          public GraphCompletion
    {
        Q_OBJECT

    public:
        /**
         * @return The single unique instance of this class.
         */
        static GraphCompletor* instance();

        /**
         * Default destructor.
         */
        virtual ~GraphCompletor();

        /**
         * Attemps to retrieve additional information from external sources
         * about resources within the given Soprano model, specified by the
         * URIs and completion mode. Any new found information is translated
         * (if necessary) and added to the given model.
         *
         * @param model The Soprano model to operate on.
         * @param completionGoals A set of URIs specifying completion goals.
         * @param targetOntologies A set of URIs specifying target ontologies.
         * @param mode What mode of completion to use.
         *
         * \sa CompletionMode
         */
        virtual void complete( Soprano::Model* model, const QSet< QUrl >& completionGoals,
                               const QSet< QUrl >& targetOntologies, CompletionMode mode );

        /**
         * Attemps to retrieve additional information from external sources
         * about resources within the given graph serialization, specified by
         * the URIs and completion mode. Any new found information is first
         * translated (if necessary) and then added to the original graph. The
         * serialization is then replaced by that of the new combined graph.
         *
         * @param serialization The RDF/XML serialization of the graph to
         *                      operate on.
         * @param completionGoals A set of URIs specifying completion goals.
         * @param targetOntologies A set of URIs specifying target ontologies.
         * @param mode What mode of completion to use.
         *
         * \sa CompletionMode
         */
        virtual void complete( QString& serialization, const QSet< QUrl >& completionGoals,
                               const QSet< QUrl >& targetOntologies, CompletionMode mode );

    private:
        friend class GraphCompletorHelper;

        /**
         * Private default constructor, object is a singleton.
         */
        GraphCompletor();

        class Private;
        Private* const d;
    };

}

#endif
