/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_GRAPHRETRIEVER_H_
#define _SEMCLIP_GRAPHRETRIEVER_H_

#include "semclip_export.h"

#include <QtCore/QObject>

class QString;
class QUrl;

namespace Soprano
{
    class Model;
}

namespace SemClip
{

    /**
     * \class GraphRetriever graphretriever.h <semclip/graphretriever.h>
     *
     * \brief Utility class for retrieving RDF graphs from Web locations.
     *
     * GraphRetriever allows retrieval of multiple RDF graph fragments from
     * the Web using HTTP GET. Target HTTP servers should support the RDF+XML
     * request MIME type for this to work.
     *
     * Clients can queue multiple get requests and must follow the last get
     * request with a call to either model() or serialization().
     *
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SEMCLIP_EXPORT GraphRetriever : public QObject
    {
        Q_OBJECT

    public:
        /**
         * Custom and default constructor.
         *
         * @param parent The parent QObject.
         */
        explicit GraphRetriever( QObject* parent = 0 );

        /**
         * Default destructor.
         */
        ~GraphRetriever();

        /**
         * Adds a get request for the given URL.
         *
         * @param url An URL specifying the location of a graph fragment to get.
         * @return True if the get request was accepted, false otherwise.
         */
        bool get( const QUrl& url );

        /**
         * Blocks, waiting for all requests to be finished, and returns a new
         * Soprano model constructed from the retrieved graph fragments. The
         * client takes over ownership of the returned model.
         *
         * @return A new Soprano model constructed from the merged fragments.
         */
        Soprano::Model* model() const;

        /**
         * Blocks, waiting for all requests to be finished, and returns the
         * serialization of the combined graph constructed from the retrieved
         * graph fragments.
         *
         * @return RDF/XML serialization of the merged fragment graph.
         */
        QString serialization() const;

        /**
         * Sets the timeout threshold. If no request finished for this amount
         * of time after calling model() or serialization(), the action will
         * abort with a timeout. A value of 0 will restore the default (8).
         *
         * @param timeoutThreshold The timeout threshold in seconds.
         */
        void setTimeoutThreshold( unsigned int timeoutThreshold );

        /**
         * Returns whether any requests were aborted due to timeouts.
         *
         * @return True if a timeout occured, false otherwise.
         */
        bool hasTimedOut() const;

    private slots:
        /**
         * Slot for QHttp requestFinished signal.
         *
         * @param id Identifies the request that has been finished.
         * @param error Indicates whether an error occured or not.
         */
        void httpRequestFinished( int id, bool error );

        /**
         * Slot for QHttp done signal.
         *
         * @param error Indicates whether an error occured or not.
         */
        void httpDone( bool error );

        /**
         * Slot for QTimer timeout signal.
         */
        void timeout();

    private:
        static const unsigned int s_defaultTimeoutThreshold;

        class Private;
        Private* const d;
    };

}

#endif
