/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_GRAPHTRANSLATOR_H_
#define _SEMCLIP_GRAPHTRANSLATOR_H_

#include "graphtranslation.h"
#include "semclip_export.h"

#include <QtCore/QObject>

namespace SemClip
{

    class GraphTranslatorHelper;
    class GraphUtility;
    class MappingElement;

    /**
     * \class GraphTranslator graphtranslator.h <semclip/graphtranslator.h>
     *
     * \brief Implements the SemClip graph translation service.
     *
     * GraphTranslator inherits from GraphTranslation and is a singleton.
     * Clients can access this class directly, but are encouraged to use the
     * SemanticClipboard facade.
     *
     * WARNING: It appears that translation does not work with Redland RDF
     *          backend models. The queries in SparqlGraphTransformer return 
     *          empty results, maybe due to buggy OPTIONAL support in Rasqal.
     *
     * \sa GraphTranslation, SemanticClipboard
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SEMCLIP_EXPORT GraphTranslator : public QObject,
                                           public GraphTranslation
    {
        Q_OBJECT

    public:
        /**
         * @return The single unique instance of this class.
         */
        static GraphTranslator* instance();

        /**
         * Default destructor.
         */
        virtual ~GraphTranslator();

        /**
         * Translates the RDF graph within the given Soprano model to the
         * supplied target ontologies. The translations are merged into the
         * given model. Overloaded versions for both a single target ontology
         * and a set of multiple target ontologies exist.
         *
         * @param model The Soprano model to operate on.
         * @param targetOntologies A set of URIs specifying target ontologies.
         */
        virtual void translate( Soprano::Model* model, const QSet< QUrl >& targetOntologies );

        /**
         * \overload
         */
        virtual void translate( Soprano::Model* model, const QUrl& targetOntology );

        /**
         * Translates the RDF graph within the given serialization to the
         * supplied target ontologies. The translations are merged into the
         * original graph, and the serialization is replaced by that of the
         * new combined graph. Overloaded versions for both a single target
         * ontology and a set of multiple target ontologies exist.
         *
         * @param serialization The RDF/XML serialization of the graph to
                                operate on.
         * @param targetOntologies A set of URIs specifying target ontologies.
         */
        virtual void translate( QString& serialization, const QSet< QUrl >& targetOntologies );

        /**
         * \overload
         */
        virtual void translate( QString& serialization, const QUrl& targetOntology );

    private:
        static const char s_translatorGroupName[];
        static const char s_blacklistGroupName[];
        static const char s_vCardRdfNamespace[];

        friend class GraphTranslatorHelper;

        /**
         * Private default constructor, object is a singleton.
         */
        GraphTranslator();

        /**
         * Queries the local mapping storage for any possible mappings between
         * source and target ontologies.
         *
         * @param sources A set of source ontologies to translate from.
         * @param targets A set of target ontologies to translate to.
         * @return A set containing any matching mappings, empty if none found.
         */
        QSet< const MappingElement* > findMappings( const QSet< QUrl >& sources, const QSet< QUrl >& targets );

        /**
         * Applies the given mapping to the supplied model.
         *
         * @param graph A GraphUtility wrapping the target model.
         * @param mapping The mapping element to apply.
         */
        void applyMapping( GraphUtility& graph, const MappingElement* mapping );

        /**
         * Returns a default list of blacklisted ontologies.
         *
         * @return The list of URIs blacklisted by default.
         */
        static QSet< QUrl > defaultOntologyBlacklist();

        /**
         * Loads a custom list of blacklisted ontologies from the config file.
         * The expected format within the blacklist group is 'key=ontologyUri',
         * where 'key' is an arbitrary value and 'ontologyUri' is the namespace
         * of the ontology to blacklist.
         *
         * @return The list of URIs blacklisted through custom config entries.
         */
        static QSet< QUrl > customOntologyBlacklist();

        /**
         * Filters out blacklisted ontologies (both default and custom set).
         *
         * @param ontologies The set of input ontology URIs.
         * @return A new set containing the non-blacklisted ontology URIs.
         */
        static QSet< QUrl > filterOntologies( const QSet< QUrl >& ontologies );

        class Private;
        Private* const d;
    };

}

#endif
