/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_GRAPHUTILITY_H_
#define _SEMCLIP_GRAPHUTILITY_H_

#include "semclip_export.h"

#include <QtCore/QUrl>
#include <QtCore/QSet>

#if QT_VERSION < 0x040700
inline uint qHash( const QUrl& url )
{
    return qHash( url.toString() );
}
#endif

class QString;

namespace Soprano
{
    class Model;
    class Node;
    class StatementIterator;
}

namespace SemClip
{

    const char uriSchema[]        = "semclip";
    const char dataDirectory[]    = "libsemclip/";
    const char configFile[]       = "libsemcliprc";

    /**
     * Extracts the namespace of an URI.
     *
     * @param uri The URI to extract the namespace from.
     * @return The namespace of that URI.
     */
    SEMCLIP_EXPORT QUrl extractNamespace( const QUrl& uri );

    /**
     * Creates a new Soprano model on a well defined backend. Used whenever
     * any component within the SemClip library needs a new model to ensure
     * usage of a common backend.
     *
     * @return A new Soprano model.
     */
    SEMCLIP_EXPORT Soprano::Model* createModel();

    /**
     * Attempts to serialize the given Soprano model to RDF/XML.
     *
     * @param model The Soprano model to serialize.
     * @return The RDF/XML serialization of the model if the serialization
     *         was successful, an empty string otherwise.
     */
    SEMCLIP_EXPORT QString serializeModelToRdfXml( const Soprano::Model* model );

    /**
     * Attempts to parse a new Soprano model from RDF/XML. Client takes
     * over ownership of the created model.
     *
     * @param serialization The RDF/XML serialization to parse.
     * @return A new Soprano model if the parsing was successful, a null
     *         pointer otherwise.
     */
    SEMCLIP_EXPORT Soprano::Model* parseModelFromRdfXml( const QString& serialization );

    /**
     * Modes of construction for createModelFromStatementIterator.
     */
    enum ConstructionMode {
        ConstructEmpty,   /**< Construct model from empty iterator. */
        ConstructNoEmpty  /**< Return null on empty iterator. */
    };

    /**
     * Constructs a new Soprano model from the given statement iterator.
     * The client takes over ownership of the created model.
     *
     * @param it The statement iterator to create the new model from.
     * @param constructEmpty Whether empty models should be created or not.
     * @return A new Soprano model or null, depending on constructEmpty
     */
    Soprano::Model* createModelFromStatementIterator( Soprano::StatementIterator& it, ConstructionMode mode = ConstructEmpty );

    /**
     * Generates an unique URI.
     *
     * @param model Soprano model to check uniqueness against.
     * @return An URI unique to the given model.
     */
    SEMCLIP_EXPORT QUrl generateUniqueUri( const Soprano::Model* model );

    /**
     * Converts all the blank nodes in a model to resource nodes with a
     * unique random generated URI. Nepomuk doesn't handle blank nodes well.
     *
     * @param model The Soprano model to modify.
     */
    SEMCLIP_EXPORT void convertBlankNodes( Soprano::Model* model );

    /**
     * \class GraphUtility graphutility.h <semclip/graphutility.h>
     *
     * \brief Utility class for shared RDF graph manipulation code.
     *
     * No detailed description available.
     *
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class GraphUtility
    {
    public:
        /**
         * Custom and default contructor.
         *
         * @param model The Soprano model to operate on.
         */
        explicit GraphUtility( Soprano::Model* model = 0 );

        /**
         * Default destructor.
         */
        ~GraphUtility();

        /**
         * Getter for the model on which the class operates.
         *
         * @return The Soprano model to operate on.
         */
        Soprano::Model* model();

        /**
         * Setter for the model on which the class operates.
         *
         * @param model The new Soprano model to operate on.
         */
        void setModel( Soprano::Model* model );

        /**
         * Merges the supplied statements into the member model.
         *
         * @param it A statement iterator to merge into the member model.
         */
        void mergeStatementIterator( Soprano::StatementIterator& it );

        /**
         * Merges the supplied statements into the member model within their
         * own context, as a consequence overwriting any existing context.
         *
         * @param it A statement iterator to merge into the member model.
         * @param context The context in which to add the new statements.
         */
        void mergeStatementIterator( Soprano::StatementIterator& it, const Soprano::Node &context );

        /**
         * Merges the contents of the supplied model into the member model.
         *
         * @param model The model to merge into the member model.
         */
        void mergeModel( const Soprano::Model* model );

        /**
         * Extracts the URIs of all ontologies used in the member model.
         *
         * @return The set of ontology URIs used in the member model.
         */
        QSet< QUrl > extractOntologies() const;

        /**
         * Identifies resources which are objects of the supplied properties.
         *
         * @param properties A set of property URIs.
         * @return All resources that are objects of the given properties.
         */
        QSet< QUrl > identifyResources( const QSet< QUrl >& properties ) const;

        /**
         * Uses the supplied ontology to infer additional facts about the
         * member model. Inferred statements are added to the member model.
         *
         * @param ontology The ontology to use for the inference.
         */
        void applyReasoner( const Soprano::Model* ontology );

    private:
        Soprano::Model* m_model;
    };

}

#endif
