/*  This file is part of the KDE semantic clipboard
    Copyright (C) 2008 Tobias Wolf <twolf@access.unizh.ch>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#ifndef _SEMCLIP_SEMANTICCLIPBOARD_H_
#define _SEMCLIP_SEMANTICCLIPBOARD_H_

#include "graphcompletion.h"
#include "graphtranslation.h"
#include "semclip_export.h"

#include <QtCore/QObject>

class QString;
class QMimeData;

namespace Soprano
{
    class Model;
}

namespace SemClip
{

    /**
     * \class SemanticClipboard semanticclipboard.h <semclip/semanticclipboard.h>
     *
     * \brief Central client access point to the SemClip library.
     *
     * SemanticClipboard fulfills two purposes. First, it acts as a facade to
     * the different subsystems of the SemClip library. Second, it acts as a
     * wrapper around the QClipboard offering convenience functions for clients
     * to directly copy and paste RDF graphs instead of having to work with the
     * QClipboard lower level MIME data functions.
     *
     * \sa GraphTranslation, GraphCompletion
     * \author Tobias Wolf <twolf@access.unizh.ch>
     */
    class SEMCLIP_EXPORT SemanticClipboard : public QObject,
                                             public GraphTranslation,
                                             public GraphCompletion
    {
        Q_OBJECT

    public:
        /**
         * Default constructor.
         */
        SemanticClipboard();

        /**
         * Default destructor.
         */
        virtual ~SemanticClipboard();

        /**
         * Clears the clipboard contents.
         */
        void clear();

        /**
         * Copies the supplied RDF/XML serialization into the clipboard.
         *
         * @param serialization The RDF/XML serialization to copy. If the
         *                      string is empty, no action is taken.
         */
        void setSerialization( const QString& serialization );

        /**
         * Adds the supplied RDF/XML serialization to the given MIME data.
         *
         * @param mimeData The MIME data object to add the serialization to.
         * @param serialization The RDF/XML serialization to add. If the
         *                      string is empty, no action is taken.
         */
        void addSerialization( QMimeData* mimeData, const QString& serialization );

        /**
         * Copies the RDF/XML serialization of the supplied Soprano model into
         * the clipboard.
         *
         * @param model The model whose RDF/XML serialization to copy. If the
         *              pointer is null, no action is taken.
         */
        void setModel( const Soprano::Model* model );

        /**
         * Adds the RDF/XML serialization of the supplied Soprano model to the
         * given MIME data.
         *
         * @param mimeData The MIME data object to add the serialization to.
         * @param model The model whose RDF/XML serialization to copy. If the
         *              pointer is null, no action is taken.
         */
        void addModel( QMimeData* mimeData, const Soprano::Model* model );

        /**
         * Retrieves a RDF/XML serialization from the clipboard.
         *
         * @return The RDF/XML serialization stored in the clipboard, or an
         *         empty string if the clipboard doesn't contain semantic data.
         */
        QString serialization() const;

        /**
         * Retrieves a RDF/XML serialization from the clipboard and constructs
         * a new Soprano model from it. The client takes over ownership of the
         * constructed model.
         *
         * @return The Soprano model constructed from the clipboard contents,
         *         or a null pointer if the clipboard doesn't contain any valid
         *         semantic data.
         */
        Soprano::Model* model() const;

        /**
         * Returns whether the clipboard holds any semantic data.
         *
         * @return True if the clipboard holds semantic data, false otherwise.
         */
        bool hasData() const;

        /**
         * Delegates to the graph translation service.
         *
         * \see GraphTranslatorInterface
         */
        virtual void translate( Soprano::Model* model, const QSet< QUrl >& targetOntologies );

        /**
         * \overload
         */
        virtual void translate( Soprano::Model* model, const QUrl& targetOntology );

        /**
         * \overload
         */
        virtual void translate( QString& serialization, const QSet< QUrl >& targetOntologies );

        /**
         * \overload
         */
        virtual void translate( QString& serialization, const QUrl& targetOntology );

        /**
         * Delegates to the graph completion service.
         *
         * \see GraphCompletorInterface
         */
        virtual void complete( Soprano::Model* model, const QSet< QUrl >& completionGoals,
                               const QSet< QUrl >& targetOntologies, CompletionMode mode );

        /**
         * \overload
         */
        virtual void complete( QString& serialization, const QSet< QUrl >& completionGoals,
                               const QSet< QUrl >& targetOntologies, CompletionMode mode );

    private:
        /**
         * Returns the MIME type used in the clipboard for the semantic data.
         *
         * @return MIME type used for the semantic data.
         */
        static QString mimeType();

        class Private;
        Private* const d;
    };

}

#endif
